
function data = load_data(problem, idx_img)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
% load_data loads the ground-truth data used for each of the recovery 
% problems investigated in the paper:
% [*] N. Eslahi and A. Foi, "Improved sparse Signal Recovery via 
%     Adaptive Correlated Noise Model", submitted to IEEE TCI, 2022.
%
% FUNCTION INTERFACE:
%        data = load_datas(problem, idx_img)
%
% ___________________________________________________________________
% INPUTS:    |      TYPE:      |  DESCRIPTION:
% -------------------------------------------------------------------
%  problem   |(char or string) | '2D Tomography'
%            |                 | '3D Tomography'
%            |                 | 'Compressive Spectral Imaging'
%            |                 | 'Compressive Temporal Imaging'
% -------------------------------------------------------------------              
%  idx_img   |  (double)       | index of underlying multispectral image 
%            |                 |    1: chart_and_stuffed_toy, 
%            |                 |    2: feathers, 
%            |                 |    3: flowers 
% (idx_img should be assigned only if problem = 'Compressive Spectral Imaging') 
%
% ___________________________________________________________________
% INPUTS:    |      TYPE:      |  DESCRIPTION:
% -------------------------------------------------------------------
%  data      | (2D/3D array)   | ground-truth image(s)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 


if strcmpi(problem,'2D Tomography') 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%%%%%%%%%%%%%%%%%%%%%%%%	2D TOMOGRAPHY  %%%%%%%%%%%%%%%%%%%%%%%%%%  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%      
    if exist('./Data/2D_MRI_experiment_data.mat','file')
        load('./Data/2D_MRI_experiment_data.mat','ImgsOrg')
        
    elseif exist('../Data/2D_MRI_experiment_data.mat','file')
           load('../Data/2D_MRI_experiment_data.mat','ImgsOrg')
        
    else
        
        fhandle = fopen('t2_icbm_normal_1mm_pn0_rf0.raws'); % the BrainWeb data
        if ~exist('t2_icbm_normal_1mm_pn0_rf0.raws','file')
            error('u:stuffed:it' , ['Could not find BrainWeb phantom file!',...
                'Please download it from \n',...
                '<a href="https://brainweb.bic.mni.mcgill.ca/brainweb/selection_normal.html">'...
                'https://brainweb.bic.mni.mcgill.ca/brainweb/selection_normal.html/</a> \n',...
                '(Modality T2, Slice thickness 1mm, ',...
                'Noise 0%%, Intensity non-uniformity 0%%)']);
        end
        width             = 181;
        height            = 217;
        depth             = 181;
        img               = fread(fhandle, width * height * depth,'uint16');
        fclose(fhandle);
        data              = reshape(img, width, height,depth);
        transverse        = "transverse slice";
        ImgsOrg(:,:,1:5)  = permute(squeeze(data(:,:,71:10:111)),[2 1 3]); % taking 5 transverse slices
        sagittal          = "sagittal slice";
        ImgsOrg(:,:,6:10) = permute(squeeze(data(71:10:111,:,:)),[2 3 1]); % taking 5 sagittal slices
        FileName          = repelem([transverse,sagittal],[5 5]);
        clear img fhandle width height depth data ans transverse sagittal
        getname = @(x) inputname(1);
        if isfolder('./Data') 
            save('./Data/2D_MRI_experiment_data.mat',  getname(FileName), getname(ImgsOrg))
        elseif isfolder('../Data')
            save('../Data/2D_MRI_experiment_data.mat', getname(FileName), getname(ImgsOrg))
        else
            mkdir Data
            save('./Data/2D_MRI_experiment_data.mat',  getname(FileName), getname(ImgsOrg))
        end
    end

    data = ImgsOrg;
    
    
    
elseif strcmpi(problem,'3D Tomography') 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%%%%%%%%%%%%%%%%%%%%%%%%	3D TOMOGRAPHY  %%%%%%%%%%%%%%%%%%%%%%%%%%  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%      
    
    if exist('./Data/3D_MRI_experiment_data.mat','file')
        load('./Data/3D_MRI_experiment_data.mat','x')
        
    elseif exist('../Data/3D_MRI_experiment_data.mat','file')
            load('../Data/3D_MRI_experiment_data.mat','x')
        
    else
        
        fhandle = fopen('t1_icbm_normal_1mm_pn0_rf0.raws'); %%% the BrainWeb data %%%
        if ~exist('t1_icbm_normal_1mm_pn0_rf0.raws','file')
            error('u:stuffed:it' , ['Could not find BrainWeb phantom file! ',...
                'Please download it from \n',...
                '<a href="http://mouldy.bic.mni.mcgill.ca/brainweb/selection_normal.html">'...
                'http://mouldy.bic.mni.mcgill.cabrainweb/selection_normal.html/</a> \n',...
                '(Modality T1, Slice thickness 1mm, ',...
                'Noise 0%%, Intensity non-uniformity 0%%)']);
        end
        width             = 181;
        height            = 217;
        depth             = 181;
        img               = fread(fhandle, width * height * depth, 'uint16');
        img               = reshape(img, width, height,depth);
        fclose(fhandle);
        
        padded_img        = zeros([217,217,217]);
        padded_img(19:181+18,:,19:181+18) = img; % zero padding to create a data of size 217x217x217
        SizeX       = [64 64 64];          % size of the downsampled volumetric data
        dwnSmpl           = SizeX(1)/217;
        padded_img        = imresize(padded_img,dwnSmpl,'bilinear'); % downsampling in the hight and width dimension
        x        = zeros(SizeX);
        for i=1:SizeX(1)
            for j=1:SizeX(2)
                x(i,j,:) = imresize(squeeze(padded_img(i,j,:)),...
                    dwnSmpl,'bilinear'); % downsampling in the depth dimension
            end
        end
        clear img padded_img dwnSmpl i j height width depth fhandle ans
        if isfolder('./Data')
            save('./Data/3D_MRI_experiment_data.mat', 'x')
        elseif isfolder('../Data')
            save('../Data/3D_MRI_experiment_data.mat', 'x')
        else
            mkdir Data
            save('./Data/3D_MRI_experiment_data.mat', 'x')
        end

    end
    
    
    data = x;
    
    
    
    
elseif strcmpi(problem,'Compressive Spectral Imaging')  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%%%%%%%%%%%%%%%%%% COMPRESSIVE SPECTRAL IMAGING %%%%%%%%%%%%%%%%%%%%  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%     

DataFiles = {'chart_and_stuffed_toy_ms_16bit.mat', 'feathers_ms_16bit.mat', 'flowers_ms_16bit.mat'};
DataNames = {'chart_and_stuffed_toy', 'feathers', 'flowers'};


if ~exist('idx_img','var') || ~ismember(idx_img,1:3)
    error(['Please check your selected "idx_img" which can take only either of {1,2,3}. \n'...
           'idx_img =  \n'...
            '                1 : %s \n' ...
            '                2 : %s \n' ...
            '                3 : %s'],DataNames{1}, DataNames{2}, DataNames{3}); 
    
end


if ~exist(DataFiles{idx_img},'file')
      error('u:stuffed:it' , ['Could not find the selected hyperspectral file! ',...
        'Please download it from \n',...
        '<a href="http://www.cs.columbia.edu/CAVE/databases/multispectral/">'...
        'http://www.cs.columbia.edu/CAVE/databases/multispectral/</a> \n',...
        '(%s)'], DataFiles{idx_img});  
end


file = load(DataFiles{idx_img});
% file_name = DataNames{idx_img};

data    = file.x(128:128+255,128:128+255,1:2:end);   % Considering only the central 256x256 spatial
                                                     % resolution of the data and every other spectral
                                                     % band (16 spectral bands, from 400nm to 700nm 
                                                     % with a 20nm interval)


clear DataFiles DataNames






elseif strcmpi(problem,'Compressive Temporal Imaging') 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%%%%%%%%%%%%%%%%%% COMPRESSIVE Temporal IMAGING %%%%%%%%%%%%%%%%%%%%  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  


if ~exist('NBA.mat','file')
      error('u:stuffed:it' , ['Could not find the selected video file! ',...
        'Please download it from \n',...
        '<a href="https://github.com/jianboyang/MMLE-GMM/">'...
        'https://github.com/jianboyang/MMLE-GMM/</a> \n',...
        '("y_org" in "Kobe40_2_gray.mat" is the ground-truth "x" in our work.)']);  
end


file = load('NBA.mat');             % NBA video of size 256x256x32
data = file.x;









else
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% ERROR %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%      
    
    error(['We do not have your selected sparse recovery problem "%s"! \n'...
        'The available problems are: \n'...
        '" 2D Tomography " : 2D Tomography \n' ...
        '" 3D Tomography " : 3D Tomography \n' ...
        '" Compressive Spectral Imaging  " : Compressive Spectral Imaging \n' ...
        '" Compressive Temporal Imaging  " : Compressive Temporal Imaging \n'],problem)
    
end






