# SPARSE SIGNAL RECOVERY UNDER ADDITIVE STATIONARY WHITE/COLORED   
# NOISE MOLDING OF DEGRADATIONS WITHIN ITERATIVE RECOVERY 
------------------------------------------------------------------- 

This package (so called varikas) contains the functions for sparse 
multidimensional and multispectral signal recovery under different 
noise modelings of degradations within recovery, as well demos.
It also contains m.files reproducing the experiments presented in 
the paper: 

[*] N. Eslahi and A. Foi, "Improved sparse Signal Recovery via  
    Adaptive Correlated Noise Model", submitted to IEEE TCI, 2022.

-------------------------------------------------------------------

Copyright (c) 2006-2022    All rights reserved.
All rights reserved.
This work should be used for nonprofit purposes only.

Authors:        Nasser Eslahi & Alessandro Foi
                 (Tampere University, Finland)

contact: firstname.lastname@tuni.fi


varikas web page:            https://webpages.tuni.fi/foi/varikas/

-------------------------------------------------------------------
 Contents
-------------------------------------------------------------------

The varikas packages includes:
./Data        : contains the ground-truth data and the set of parameters 
                used in the simulations of [*].
        ./Data/load_data.m   : loads the ground-truth data
        ./Data/load_params.m : loads the parameters used for the simulations in [*]
        ./Data/chart_and_stuffed_toy_ms_16bit.mat : 512x512x32 multispectral toy image
        ./Data/feathers_ms_16bit.mat              : 512x512x32 multispectral feathers image
        ./Data/flowers_ms_16bit.mat               : 512x512x32 multispectral flowers image
        ./Data/NBA.mat                            : 256x256x32 NBA video
        ./Data/t1_icbm_normal_1mm_pn0_rf0.raws    : 3D BrainWeb T1 phantom
        ./Data/t2_icbm_normal_1mm_pn0_rf0.raws    : 3D BrainWeb T2 phantom


./Denoisers   : contains the BMxD filter used within the PnP frameworks.
        ./Denoisers/BM3D/   : BM3D package
        ./Denoisers/BM4D/   : BM4D package
        ./Denoisers/RF3D/   : RF3D package

./Reproducible: contains the files to reproduce the Tables 1,2,3 and Figure 6 of [*]. 
        ./Reproducible/Reproduce_Fig_6__Compressive_Temporal_Imaging.m
        ./Reproducible/Reproduce_Table_I__Compressive_Spectral_Imaging.m
        ./Reproducible/Reproduce_Table_II__2D_Tomography.m
        ./Reproducible/Reproduce_Table_III__3D_Tomography.m

 
./Results     : an empty folder in the directory to save the final recovery results.

./Utilities   : contains the functions and dependencies used in acquisition, 
                recovery and visualization of the results.

        ./Utilities/Acquisition.m              : for acquiring the measurements.
        ./Utilities/Est_effective_noise_PSD.m  : computes the approximated effective noise PSD.
        ./Utilities/Est_rootPSD.m              : computes the root-PSD via a local moving window.
        ./Utilities/Est_std.m                  : computes the AWGN std from the noisy signal.
        ./Utilities/FFT_PSD_domain_convertor.m : converts a global FFT-PSD into a transform-domain PSD.
        ./Utilities/FFT_PSD_Upsampling.m       : upsamples the local FFT-PSD to constitute a global FFT-PSD.
        ./Utilities/PSD_Auxiliaries.m          : computes the PSD compensation ratio and 
                                                 the local FFT-root-PSD of back-projected measurement error.
        ./Utilities/Visualization.m            : illustrates the recovered images/video as well as 
                                                 the oracle effective noise root-PSD.



Demo_2D_Tomography.m : a demo for 2D tomography
Demo_3D_Tomography.m : a demo for 3D tomography
Demo_Compressive_Spectral_Imaging.m : a demo for compressive hyperspectral recovery
Demo_Compressive_Temporal_Imaging.m : a demo for compressive video recovery

Iterative_PnP.m : the main function which performs iterative sparse signal recovery 
                  within a PnP framework (e.g., ADMM, ISTA, or FISTA).


For a detailed information on each function, please check its syntax.

-------------------------------------------------------------------
 Disclaimer
-------------------------------------------------------------------

The code is available for non-commercial use only. For details, see LICENSE.

-------------------------------------------------------------------
 Feedback
-------------------------------------------------------------------
 
If you have any comment, suggestion, or question, please do contact
 Nasser Eslahi  < nasser.eslahi _at_ tuni.fi >
