%%               An Auxiliary for Visualization


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%%%%%%%%%%%%%%%%%%%%%%%%	2D TOMOGRAPHY  %%%%%%%%%%%%%%%%%%%%%%%%%%  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  

if strcmpi(param.problem,'2D Tomography')


fg=figure(777);
fg.WindowState = 'Maximized';  % making the whole figure as size as your screen


%%% Drawing the evolution of PSNR(dB)/MSRE(dB) %%%
subplot(2,5,1:2)
plot(info_rec,'r','linewidth',2); grid on
set(gca,'Xlim',[0 k]);
xlabel('Iteration')
ylabel(sprintf('%s (dB)',param.assmt))
set(gca,'TickLabelInterpreter','latex')

drawnow
pause(eps)


%%% Illustrating the effective noise root-PSD %%%
if k==1
    sigma_kk = sigma_e_1;       % the first iteration is modeled as AWGN 
else
    if  sum(strcmpi(thr_setting,{'C','C*'})) % for correlated noise modeling
        sigma_kk = -inf;        % we show the effective noise root-PSD
    else 
        sigma_kk = sigma_e_k;   % visualizing the adopted/estimated noise std ...
                                % alongside the effective noise root-PSD
    end
end


if param.compute_PSNR
    %%% oracle effective noise root-PSD %%%
    rootPSD_e_k = Est_rootPSD(z_k - x, param.Tr_Sz, param.HT_Tr, 0, 'sstd');
    
    %%% visualize the computed root-PSD of effective noise %%%
    subplot(2,5,4)
    txt_eff_rPSD = ['$\boldmath{\sigma}_{\mathcal{T}_{2D}(\mathbf{e}_{' num2str(k) '})}$'];
    bar_gray(rootPSD_e_k, txt_eff_rPSD, sigma_kk);
    clear e_k rootPSD_e_k txt_eff_rPSD
end

if sum(strcmpi(thr_setting,{'C','C*'}))
    %%% visualize the approximated root-PSD of effective noise %%%
    subplot(2,5,5)
    txt_approx_rPSD = ['$\hat{\boldmath{\sigma}}_{\mathcal{T}_{2D}(\mathbf{e}_{' num2str(k) '})}$'];
    bar_gray(rootPSD_HT, txt_approx_rPSD, sigma_kk);
    clear sigma_kk txt_approx_rPSD
end       


h   = subplot(2,5,6:10);
pos = get(h,'Position');
set(h,'outerposition',[pos(1) pos(2)-.05 pos(3:4)]);
clear pos
set(gcf,'color',[1 1 1])

%%% Visualizing the recovered slices %%%

if param.compute_PSNR
    clip = @(Z) max(min(x(:)),min(max(x(:)),Z)); % clipping function only for visualization
    
    imshow([fftshift(mask).*max(x(:)), clip(param.x_0), clip(z_k), clip(x_k), x]...
        ,[],'InitialMagnification','fit');
else
    clip = @(Z) max(min(x_k(:)),min(max(x_k(:)),Z)); % clipping function only for visualization
    
    imshow([fftshift(mask).*max(x_k(:)), clip(param.x_0), clip(z_k), clip(x_k)]...
        ,[],'InitialMagnification','fit');
end

str_title = sprintf(['%s.\n Recovery%s from %0.1f%% %s sampling of the FFT spectra\n',...
    'via %s PnP-%s, under {\\color{blue}%s} threshold setting with \\lambda=%0.1f\n %s\n', ...
    'Iteration #{\\color{blue}%02d} (out of %02d), %s = {\\color{blue}%0.2f} dB\n'], param.problem, param.file_name, ...
    mean(param.mask(:))*100, strrep(param.samp_strat, '_' ,' '), param.denoiser,...
    param.PnP_framework, param.thr_setting, lambda_thr, param.txt_snr, ...
    k, param.k_final, param.assmt, info_rec(k));
title(str_title,'HorizontalAlignment','center','VerticalAlignment','bottom','interpreter', 'tex');

text(param.SizeX(2)*0.5,0*param.SizeX(1)-(1.5),'$\textrm{Sampling Mask}$','Interpreter','Latex',...
    'HorizontalAlignment','center','VerticalAlignment','bottom');
text(param.SizeX(2)*1.5,0*param.SizeX(1)-(1.5),'$\mathbf{x}_{0}\!=\!\mathbf{M}^{\dagger}\mathbf{y}$','Interpreter','Latex',...
    'HorizontalAlignment','center','VerticalAlignment','bottom');
text(param.SizeX(2)*2.5,0*param.SizeX(1)-(1.5),'$\mathbf{z}_{k}$','Interpreter','Latex',...
    'HorizontalAlignment','center','VerticalAlignment','bottom');
text(param.SizeX(2)*3.5,0*param.SizeX(1)-(1.5),'$\mathbf{x}_{k}$','Interpreter','Latex',...
    'HorizontalAlignment','center','VerticalAlignment','bottom');
if param.compute_PSNR
text(param.SizeX(2)*4.5,0*param.SizeX(1)+(1.5),'$\mathbf{x}$','Interpreter','Latex',...
    'HorizontalAlignment','center','VerticalAlignment','bottom');
end

pause(eps)

clear sigma_kk clip fg h legend_handle str_title





end







%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%%%%%%%%%%%%%%%%%%%%%%%%	3D TOMOGRAPHY  %%%%%%%%%%%%%%%%%%%%%%%%%%  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 


if strcmpi(param.problem,'3D Tomography')

%%% Visualizing the 3D cross-section of the recovered image %%%


if k==param.iter_show(1)
   fprintf(['We can show only the outer skin of the 3D root-PSDs, and separately display\n'...
            'the three  lowest-frequency faces (LLL corner) and \n' ...
            'the three highest-frequency faces (HHH corner).\n']);    
end
                                    

    if param.compute_PSNR
    %%% oracle effective noise root-PSD %%% 
    rootPSD_e_k = Est_rootPSD(z_k - x, param.Tr_Sz, param.HT_Tr, 0, 'sstd');  % DCT root-PSD of e_k using an oracle
    %%% viusalizing the effective noise root-PSD %%% 
    fg1=figure(1); fg1.WindowState = 'normal';  movegui(fg1,'northeast');
    CubeFaceVis(rootPSD_e_k);
    title(sprintf('oracle effective noise root-PSD $\\boldmath{\\sigma}_{\\mathcal{T}_{3D}(\\boldmath{e}_{%d})}$\n',k),...
        'Interpreter','Latex','fontsize',14,'Position',[-14 -10])
    clear rootPSD_e_k
    end
    
    if sum(strcmpi(thr_setting,{'C','C*'}))
        fg2=figure(2); fg2.WindowState = 'normal'; movegui(fg2,'northwest');
        CubeFaceVis(rootPSD_HT); % approximated DCT root-PSD (given to the filter)
        title(sprintf('approximated root-PSD $\\boldmath{\\hat{\\sigma}}_{\\mathcal{T}_{3D}(\\boldmath{e}_{%d})}$\n',k),...
            'Interpreter','Latex','fontsize',14,'Position',[-14 -10])
    end
                                 
                                    
                                    

   
fg=figure(777);
fg.WindowState = 'normal'; % 'Maximized';  % making the whole figure as size as your screen
movegui(fg,'south');

subplot(1,5,1)
visualize3DXsect(fftshift(param.mask))
title('$\textrm{3D sampling mask}$','Interpreter','Latex','fontsize',12);

subplot(1,5,2)
visualize3DXsect(param.x_0)
title('$\mathbf{x}_{0}\!=\!\mathbf{M}^{\dagger}\mathbf{y}$','Interpreter','Latex','fontsize',14);

subplot(1,5,3)
visualize3DXsect(z_k)
title('$\mathbf{z}_{k}$','Interpreter','Latex','fontsize',14);

subplot(1,5,4)
visualize3DXsect(x_k)
title('$\mathbf{x}_{k}$','Interpreter','Latex','fontsize',14);

if param.compute_PSNR
subplot(1,5,5)
visualize3DXsect(x)
title('$\mathbf{x}$','Interpreter','Latex','fontsize',14);
end


str_title = sprintf(['\n\n\n\n\n%s.\n Recovery%s from %0.2f%% %s sampling of the FFT spectra\n',...
    'via %s PnP-%s, under {\\color{blue}%s} threshold setting with \\lambda=%0.1f\n %s\n', ...
    'Iteration #{\\color{blue}%02d} (out of %02d), %s = {\\color{blue}%0.2f} dB\n'], param.problem, param.file_name, ...
    mean(param.mask(:))*100, strrep(param.samp_strat, '_' ,' '), param.denoiser,...
    param.PnP_framework, param.thr_setting, lambda_thr, param.txt_snr, ...
    k, param.k_final, param.assmt, info_rec(k));

suptitle(str_title); hold off
drawnow
pause(eps)

clear clip fg str_title fg1 fg2


end












%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%%%%%%%%%%%%%%%%%   COMPRESSIVE SPECTRAL IMAGING  %%%%%%%%%%%%%%%%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 


if strcmpi(param.problem,'Compressive Spectral Imaging')
    
    
if k==param.iter_show(1)
   fprintf(['We can show only the outer skin of the 3D root-PSDs, and separately display\n'...
            'the three  lowest-frequency faces (LLL corner) and \n' ...
            'the three highest-frequency faces (HHH corner).\n']);   
end
                                                                                                            

if param.compute_PSNR 
    %%% Illustrating the effective noise root-PSD %%%  
    loc_FFT_rootPSD_e_k = Est_rootPSD(z_k - x, param.locWin,...
                           {'fft','fft','fft'}, 0, 'sstd');  % DCT root-PSD of eta using an oracle
    % Resizing (interpolating) 31x31x13 FFT PSD to become 31x31x16 FFT PSD
    glob_FFT_PSD_e_k = abs(reshape(...
                        imresize(...
                             reshape(loc_FFT_rootPSD_e_k.^2,[prod(param.locWin(1:2)) param.locWin(3)]),...
                        [prod(param.locWin(1:2)) param.SizeX(3)])*(param.SizeX(3)/param.locWin(3)),...
                    [param.locWin(1:2) param.SizeX(3)]));

    %%% converting the global FFT-PSD into HT_Tr-root-PSD compatible with filter
    %%% (i.e. 8x8x16 DCT root-PSD for hard-thresholding stage of BM4D in this experiment)
    rootPSD_e_k     = sqrt(FFT_PSD_domain_convertor(glob_FFT_PSD_e_k, param.Tr_Sz, param.HT_Tr));
    fg1=figure(1); fg1.WindowState = 'normal';  movegui(fg1,'northeast');
    CubeFaceVis(rootPSD_e_k);
    title(sprintf('oracle effective noise root-PSD $\\boldmath{\\sigma}_{\\mathcal{T}_{3D}(\\boldmath{e}_{%d})}$\n',k),...
        'Interpreter','Latex','fontsize',14,'Position',[-40 -30]), drawnow
    pause(.1)
    clear loc_FFT_rootPSD_e_k glob_FFT_PSD_e_k rootPSD_e_k
end


if sum(strcmpi(thr_setting,{'C','C*'}))
    fg2=figure(2); fg2.WindowState = 'normal'; movegui(fg2,'northwest');
    CubeFaceVis(rootPSD_HT); % approximated DCT root-PSD (given to the filter)
    title(sprintf('approximated root-PSD $\\boldmath{\\hat{\\sigma}}_{\\mathcal{T}_{3D}(\\boldmath{e}_{%d})}$\n',k),...
                  'Interpreter','Latex','fontsize',14,'Position',[-40 -30]), drawnow
    pause(.1)
end
                                  
                                   
%%% Visualizing the RGB channels of the recovered image %%%                                        
fg=figure(777);
fg.WindowState = 'normal';  % making the whole figure as size as your screen
movegui(fg,'south');
if ~isfield(param,'Spctr_bands') || isempty(param.Spctr_bands)
    param.Spctr_bands = linspace(400,700,param.SizeX(3));
end

subplot(3,1,2:3)

if param.compute_PSNR 
    if ~exist('RGB_x','var')&& ~exist('RGB_x_0','var')
        gammaCorr = .5; % gamma correction factor for visualization
        clip      = @(Z) max(min(x(:)),min(max(x(:)),Z));   % clipping function only for visualization
        RGB_x     = MS2RGB(x, param.Spctr_bands).^gammaCorr;
        RGB_x_0   = MS2RGB(clip(pinvM(y)), param.Spctr_bands).^gammaCorr;
    end
    imshow([RGB_x_0,...
            MS2RGB(clip(z_k), param.Spctr_bands).^gammaCorr,...
            MS2RGB(clip(x_k), param.Spctr_bands).^gammaCorr,...
            RGB_x]); 
        
else
    if ~exist('RGB_x_0','var')
        gammaCorr = .5; % gamma correction factor for visualization
        clip      = @(Z) max(min(x_k(:)),min(max(x_k(:)),Z));   % clipping function only for visualization
        RGB_x_0   = MS2RGB(clip(pinvM(y)), param.Spctr_bands).^gammaCorr;
    end
    imshow([RGB_x_0,...
            MS2RGB(clip(z_k), param.Spctr_bands).^gammaCorr,...
            MS2RGB(clip(x_k), param.Spctr_bands).^gammaCorr]);     
    
end


    text(param.SizeX(2)*0.5,0*param.SizeX(1)-(1.5),'$\mathbf{M}^{\dagger}\mathbf{y}$',...
        'Interpreter','Latex',...
        'HorizontalAlignment','center','VerticalAlignment','bottom','fontsize',16);
    text(param.SizeX(2)*1.5,0*param.SizeX(1)-(1.5),'$\mathbf{z}_{k}$','Interpreter','Latex',...
        'HorizontalAlignment','center','VerticalAlignment','bottom','fontsize',16);
    text(param.SizeX(2)*2.5,0*param.SizeX(1)-(1.5),'$\mathbf{x}_{k}$','Interpreter','Latex',...
        'HorizontalAlignment','center','VerticalAlignment','bottom','fontsize',16);
    if param.compute_PSNR 
    text(param.SizeX(2)*3.5,0*param.SizeX(1)+(1.5),'$\mathbf{x}$','Interpreter','Latex',...
        'HorizontalAlignment','center','VerticalAlignment','bottom','fontsize',16);
    end

    
    title(sprintf(['%s.\n Iterative sparse recovery of %s from \n',...
                   'a single-shot spectrally compressed measurements \n',...
                   'via %s PnP-%s, under {\\color{blue}%s} setting with λ=%0.1f \n'...
                   '%s\n Iteration #{\\color{blue}%02d} (out of %02d), %s = {\\color{blue}%0.2f} dB\n'],...
                   param.problem, param.file_name, param.denoiser,...
                   param.PnP_framework , param.thr_setting, lambda_thr,...
                   param.txt_snr, k, param.k_final, param.assmt, info_rec(k)),... 
                   'Interpreter','tex','fontsize',14,...
                   'HorizontalAlignment','center','VerticalAlignment','bottom');
drawnow
pause(eps)


if k==param.iter_show(end)
   clear RGB_x RGB_x_0 gammaCorr clip fg fg1 fg2
end

end









%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%%%%%%%%%%%%%%%%%   COMPRESSIVE TEMPORAL IMAGING  %%%%%%%%%%%%%%%%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 


if strcmpi(param.problem,'Compressive Temporal Imaging')
    %%% Visualizing one of the recovered frames of the video %%%
    
    fram_to_show = max(1, round(param.SizeX(3)/(param.SizeY(3)+1))); % the frame# to show
    
    if param.compute_PSNR
        %%% computing FPN and RND components of oracle effective noise %%%
        e_k = z_k - x; %%% oracle effective noise %%%
        e_k_FPN = mean(e_k,3); % FPN component of e_k
        e_k_RND = e_k - mean(e_k,3); % END component of e_k
        
        %%% computing FPN and RND DCT-root-PSDs of effective noise %%%
        oracle_rootPSD_e_k_FPN = Est_rootPSD(e_k_FPN, param.Tr_Sz, param.HT_Tr, 0, 'sstd');
        oracle_rootPSD_e_k_RND = Est_rootPSD(e_k_RND, param.Tr_Sz, param.HT_Tr, 0, 'sstd');
        clear e_k e_k_FPN e_k_RND
    end
        

    if k==1
        sigma_kk_rnd = rootPSD_HT(end);     % the first iteration is modeled as AWGN
        sigma_kk_fpn = 0; % no FPN
    else
        if ~sum(strcmpi(thr_setting,{'C','C*'}))
            sigma_kk_rnd = rootPSD_HT(end); % visualizing the adopted/estimated noise s.t.d.
                                            % alongside the effective noise root-PSD
            sigma_kk_fpn = 0;    % no FPN
        else
            sigma_kk_rnd = -inf; % we show the effective noise root-PSD
            sigma_kk_fpn = -inf; % we show the effective noise root-PSD
        end
        
    end

    %%% Illustrating the effective noise root-PSDs %%%    
    fg=figure(888);
    fg.WindowState = 'Maximized';
    
    if param.compute_PSNR
        %%% visualize the computed FPN and RND DCT-root-PSDs of effective noise %%%
        subplot(3,4,7)
        txt = ['$\boldmath{\sigma}_{\mathcal{T}_{2D}(\mathbf{e}_{FPN_{' num2str(k) '}})}$'];
        bar_gray(oracle_rootPSD_e_k_FPN, txt, sigma_kk_fpn);
        
        subplot(3,4,8)
        txt = ['$\boldmath{\sigma}_{\mathcal{T}_{2D}(\mathbf{e}_{RND_{' num2str(k) '}})}$'];
        bar_gray(oracle_rootPSD_e_k_RND, txt, sigma_kk_rnd);
        clear oracle_rootPSD_e_k_FPN oracle_rootPSD_e_k_RND
    end
    
    if sum(strcmpi(thr_setting,{'C','C*'})) && k>1
        %%% visualize the approximated FPN and RND DCT-root-PSDs %%%
        subplot(3,4,11)
        txt = ['$\hat{\boldmath{\sigma}}_{\mathcal{T}_{2D}(\mathbf{e}_{FPN_{' num2str(k) '}})}$'];
        bar_gray(rootPSD_HT(:,:,1), txt, sigma_kk_fpn);
        
        subplot(3,4,12)
        txt = ['$\hat{\boldmath{\sigma}}_{\mathcal{T}_{2D}(\mathbf{e}_{RND_{' num2str(k) '}})}$'];
        bar_gray(rootPSD_HT(:,:,2), txt, sigma_kk_rnd);
    end
    
    subplot(3,4,2:4)
    axis off
   title(sprintf(['%s.\n Iterative sparse recovery of %dx%dx%d %s from \n',...
                   '%dx%dx%d temporally compressed measurements \n',...
                   'via %s PnP-%s, under {\\color{blue}%s} setting with λ=%0.1f \n'...
                   '%s\n Iteration #{\\color{blue}%02d} (out of %02d), %s = {\\color{blue}%0.2f} dB\n'],...
                   param.problem, param.SizeX(1), param.SizeX(2), param.SizeX(3),...
                   param.file_name, param.SizeY(1), param.SizeY(2), param.SizeY(3),...
                   param.denoiser, param.PnP_framework, param.thr_setting, lambda_thr, ...
                   param.txt_snr, k, param.k_final, param.assmt, info_rec(k)),... 
                   'Interpreter','tex','fontsize',14,...
                   'HorizontalAlignment','center','VerticalAlignment','bottom','Position',[.5 0]);
    

    
    
    subplot(3,4,1)
    imshow(y(:,:,ceil(param.SizeY(3)*fram_to_show/param.SizeX(3))),[]), colorbar, axis tight equal
    title('$\mathbf{y}$','Interpreter','Latex','fontsize',12);
    
    subplot(3,4,5)
    imshow(param.x_0(:,:,fram_to_show),[]), colorbar, axis tight equal
    title('$\mathbf{x}_{0}\!=\!\mathbf{M}^{\dagger}\mathbf{y}$','Interpreter','Latex','fontsize',14);
    
    subplot(3,4,6)
    imshow(z_k(:,:,fram_to_show),[]), colorbar, axis tight equal
    title('$\mathbf{z}_{k}$','Interpreter','Latex','fontsize',14);
    
    subplot(3,4,9)
    imshow(x_k(:,:,fram_to_show),[]), colorbar, axis tight equal
    title('$\mathbf{x}_{k}$','Interpreter','Latex','fontsize',14);
    
    if param.compute_PSNR
    subplot(3,4,10)
    imshow(x(:,:,fram_to_show),[]), colorbar, axis tight equal
    title('$\mathbf{x}$','Interpreter','Latex','fontsize',14);
    end
    
    drawnow
    pause(eps)
    
    clear txt  fg  frame_to_show  sigma_kk_fpn sigma_kk_rnd
       
end




%%
%% Auxiliary function for visualization
%%% Note that you can extract each of the following functions as ...
%%% a separate function and illustrate your own data!
%%%
%%%% bar_gray         : draws a 3D bar graph
%%%% CubeFaceVis      : shows the inner and outer skin of a 3D data
%%%% visualize3DXsect : shows a cross-section of a 3D data
%%%% MS2RGB           : shows an RGB representation of a 3D hyperspectral image
%%%% suptitle         : adds a title above the grid of subplots (one can use ... 
%%%%                    sgtitle instead of suptitle if MATLAB 2018b is installed)
%%



function bar_gray(rootPSD, txt, sigma_k)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  bar_gray draws a three-dimensional bar graph of the input root-PSD
%
%  FUNCTION INTERFACE:
%  bar_gray(rootPSD, txt, sigma_k)
% 
%  INPUTS:
%     1) rootPSD          (2D array) : the input root-PSD
%     2) txt              (string)   : the text to be displayed as the title
%     3) sigma_k          (double)   : the estimated sigma-value at the HH frequency 
%                                      if sigma_k = [] or inf, it isn't displayed
%-------------------------------------------------------------------------
%  EXAMPLES:
%  prerequisite : Est_rootPSD.m (to compute the local Tr-PSD using the sample variance)
%
%   1) correlated noise
%
%       I        = im2double(imread('cameraman.tif'));
%       w        = randn(size(I)); % white Gaussian noise
%       g1       = fspecial('gaussian',7,1.5);
%       g2       = fspecial('gaussian',7,0.5);
%       g        = (g1-g2)/4;    % correlation kernel
%       eta      = imfilter(w,g,'circular');   % correlated noise
%       z        = I + eta;  % noisy image
%       
%       dct_rootPSD_eta = Est_rootPSD(eta, [8 8], {'dct','dct'}, 0, 'sstd');  % estimated DCT root-PSD of eta
%       sigma_HH = Est_std(z,[8 8], {'dct','dct'}, 'MAD'); % estimated noise power at HH frequency 
%       txt      = '$\boldmath{\sigma}_{\mathcal{T}_{2D}(\boldmath{\eta})}$';
%       
%       figure, bar_gray(dct_rootPSD_eta, txt, sigma_HH) % visualization with indicator
%       figure, bar_gray(dct_rootPSD_eta, txt) % visualization without indicator
%      
%
%   2) white noise
%
%       I        = im2double(imread('cameraman.tif'));
%       sigma    = rand; % noise standard deviation
%       eta      = sigma*randn(size(I)); % white Gaussian noise  
%       z        = I + eta;
%       dct_rootPSD_eta = Est_rootPSD(eta, [8 8], {'dct','dct'}, 0, 'sstd');  % estimated DCT root-PSD of eta
%       sigma_HH = Est_std(z,[8 8], {'dct','dct'}, 'MAD'); % estimated noise standard deviation
%       txt      = '$\boldmath{\sigma}_{\mathcal{T}_{2D}(\boldmath{\eta})}$';
%       figure, bar_gray(dct_rootPSD_eta, txt, sigma_HH) % visualization
%
%-------------------------------------------------------------------------
%            Authors :         Nasser Eslahi & Alessandro Foi
%                                   Tampere University
%        Contact info:          firstname.lastname@tuni.fi      

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if ~exist('txt','var')
    txt='$\boldmath{\sigma}_{\mathcal{T}_{2D}(\boldmath{\eta})}$';
end

if ~exist('sigma_k','var')||isempty(sigma_k)|| isinf(sigma_k)
    sigma_k = -inf;
end

[s1, s2] = size(rootPSD);
bar3(rootPSD);
view([61,31]);colormap(0.9*ones(64,3));
title(txt,'Interpreter','latex','fontsize',21)
    
axis tight square, camproj('persp'),
box on,  set(gca,'color',[1,1,1])
xticks([1 s1]); xticklabels([0 s1-1]);
yticks([1 s2]); yticklabels([0 s2-1]);
        
camproj('orth')
set(gcf,'color',[1 1 1])
box on, grid on
view(-40,25)
cl1=camlight(-230,135);
cl2=camlight(0,5);
cl1.Color=[1 1 1]*1;
cl2.Color=[1 1 1]*0.6;
set(gca,'xlim',[0.5,8.5])
set(gca,'ylim',[0.5,8.5])
set(gca,'Clipping','off')
if sigma_k ~= -inf
    hold on
    sd=0.5;
    ph=plot3(8.5+sd,8.5+sd,sigma_k,'<');
    ph.MarkerFaceColor=[0 0 0];
    ph.MarkerSize=21;
    ph.MarkerEdgeColor=[0 0 0];
    hold off
end
lighting gouraud
% drawnow
set(gca,'TickLabelInterpreter','tex') %'latex'

ax=gca;
ax.XGrid = 'off';
ax.YGrid = 'off';
ax.LineWidth = 1.5;
ax.GridAlpha = 0.7;
set(gca,'DataAspectRatio',[1.3 1.3 max(rootPSD(:))/10])

set(gca,'TickLabelInterpreter','latex')
set(gca,'FontSize',21)
set(gca,'ZMinorGrid','on');
Ch=get(gca,'Children');
lighting flat  % probably not needed
if sigma_k == -inf
    set(Ch(1),'Position',get(Ch(1),'Position').*[1 1.16 2])
    set(Ch(2),'Position',get(Ch(2),'Position').*[1 2.3 1])
    set(Ch(1),'Color',[1 1 1]*0.60)
    set(Ch(2),'Color',[1 1 1]*0.85)
else
    set(Ch(2),'Position',get(Ch(2),'Position').*[1 1.16 2])
    set(Ch(3),'Position',get(Ch(3),'Position').*[1 2.3 1])
    set(Ch(2),'Color',[1 1 1]*0.60)
    set(Ch(3),'Color',[1 1 1]*0.85)
end

end








function CubeFaceVis(rootPSD)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% We can show only the outer skin of a 3D root-PSD, and separately display 
% the three lowest-frequency faces that stem from the DC (LLL corner, at the back) 
% and the three highest-frequency faces (HHH corner, diametrically opposed to the DC)
%
% CubeFaceVis visualizes the outer and inner faces of a 3D cube
%
%-------------------------------------------------------------------------
%  EXAMPLES:
%  prerequisite : Est_rootPSD.m (to compute the local Tr-PSD using the sample variance)
%
%   1) correlated noise
%
%       load mri;  % load 3D MRI data
%       D        = double(squeeze(D));  % of size 128x128x27
%       w        = randn(size(D));      % white Gaussian noise
%       g        = rand(8,7,4);         % correlation kernel (random)
%       eta      = imfilter(w,g,'circular');   % correlated noise
%       z        = D + eta;  % noisy image
%       
%       dct_rootPSD_eta = Est_rootPSD(eta, [8 8 8], {'dct','dct', 'dct'}, 0, 'sstd');  % estimated DCT root-PSD of eta
%   
%       CubeFaceVis(dct_rootPSD_eta);   % visualization
%      
%
%   2) white noise
%
%       load mri;  % load 3D MRI data
%       D        = double(squeeze(D));  % of size 128x128x27
%       sigma    = max(D(:))*rand;      % noise standard deviation 
%       eta      = sigma*randn(size(D));      % white Gaussian noise
%       z        = D + eta;  % noisy image
%       
%       dct_rootPSD_eta = Est_rootPSD(eta, [8 8 8], {'dct','dct', 'dct'}, 0, 'sstd');  % estimated DCT root-PSD of eta
%       sigma_HHH = Est_std(z,[8 8 8], {'dct','dct','dct'}, 'MAD'); % estimated noise standard deviation  
%
%       CubeFaceVis(dct_rootPSD_eta);   % visualization
%
%-------------------------------------------------------------------------
%            Authors :         Nasser Eslahi & Alessandro Foi
%                                   Tampere University
%        Contact info:          firstname.lastname@tuni.fi      

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% extrapolation of 3D cube w.r.t its outer faces (top, right, front)
rPSD = zeros(size(rootPSD)+1);
rPSD(1:end-1,1:end-1,1:end-1) = rootPSD;
rPSD(end,1:end-1,1:end-1)     = rootPSD(end,:,:);
rPSD(1:end-1,end,1:end-1)     = rootPSD(:,end,:);
rPSD(:,:,end)                 = rPSD(:,:,end-1);
rPSD(end,end,:)               = rPSD(end-1,end-1,:);
rPSD = permute(rPSD,[2 1 3]);


cMap = parula; % you can choose other color maps, e.g., jet, hot


fResponse    = @(x) log10(x); invfResponse = @(x) 10.^x;  % logarithmic scale for illustration
% fResponse  = @(x) x; invfResponse = @(x) x;  
rPSD_scale   = fResponse(rPSD); % scaled root-PSD

[x1, x2, x3] = size(rPSD);
[x, y, z]    = meshgrid(1:x1, 1:x2, 1:x3);


sq123{1} = [0 0 1 1 0;0 1 1 0 0;0 0 0 0 0];
sq123{2} = [0 0 1 1 0;0 1 1 0 0;1 1 1 1 1];
sq123{3} = sq123{1}([1 3 2],:);
sq123{4} = sq123{2}([1 3 2],:);
sq123{5} = sq123{1}([3 1 2],:);
sq123{6} = sq123{2}([3 1 2],:);
sPidx    = [inf inf 1];
sPidxB   = [0 0 1];

for j=1:2
    
    % %%% maximizing the screen
    % set(groot, 'defaultFigureWindowState', 'maximized');
    % set(get(handle(gcf),'JavaFrame'),'Maximized',1);
    
    if j==1
        ax1=subplot(1,2,j,'align');
        %%% inner faces of cube (left, rear, bottom)
        xslice = 1;
        yslice = 1;
        zslice = 1;
        hold on
        mHandle=plot3(1.2,1.2,1.2,'k.');
        set(mHandle,'markerSize',28);
    elseif j==2 
        ax2=subplot(1,2,j,'align');
        %%% outer faces of cube (right, front, top)
        xslice = x1;
        yslice = x2;
        zslice = x3;
        hold on
        mHandle=plot3(size(rPSD,1),size(rPSD,2),size(rPSD,3),'k.');
        set(mHandle,'markerSize',28);
    end
    
       slice(x,y,z,rPSD_scale,xslice,yslice, zslice);
%     h = slice(xx,yy,zz,fResponse((rPSD)),xslice,yslice, zslice);
%     pFaceAlpha = .7;
%     s.FaceAlpha = 'interp' ; 
%     s.FaceColor = 'flat';
%     set(h,'FaceAlpha',pFaceAlpha);
%     set(h, 'EdgeColor', 'none');

    axis vis3d;
    axis equal
    axis off
    
    %     set(gcf,'color',[1 1 1])
    %     set(h,'EdgeColor',[0 0 0]); set(h,'EdgeAlpha',0.1);
    %     camproj('persp')

    xticks([]);
    yticks([]);
    zticks([]);
    
    view([120 25]);
    set(gcf,'color',[1,1,1])
    colormap(cMap)
    cb=colorbar;
    
    if size(rPSD,3)>20
       cbTicks = 7;   
    else
       cbTicks = 5; 
    end
    
    cbL=get(cb,'Limits');
    set(cb,'ticks',linspace(cbL(1),cbL(2),cbTicks));
    set(cb,'TickLabels',num2str(invfResponse(linspace(min(rPSD_scale(:)),max(rPSD_scale(:)),cbTicks)'),2))
    

    if j==1
         set(cb,'Visible','off');
        for jj=1:3
            pHandle=patch(max(sq123{1}(1,:)*size(rPSD,1),1),max(sq123{1}(2,:)*size(rPSD,2),1),max(sq123{1}(3,:)*size(rPSD,3),1),1);
            set(pHandle,'faceColor','none','EdgeColor',[0 0 0],'EdgeAlpha',1,'LineWidth',3);
            pHandle=patch(max(sq123{1}(1,:)*size(rPSD,1)+sPidxB(1)*size(rPSD,1),1),max(sq123{1}(2,:)*size(rPSD,2)+sPidxB(2)*size(rPSD,2),1),max(sq123{1}(3,:)*size(rPSD,3)+sPidxB(3)*size(rPSD,3),1),1);
            set(pHandle,'faceColor','none','EdgeColor',[0 0 0],'EdgeAlpha',1,'LineWidth',.7);
            pHandle=patch(max(sq123{1}(1,:)*size(rPSD,1),1),max(sq123{1}(2,:)*size(rPSD,2),1),min(max(sq123{1}(3,:)*size(rPSD,3),1),1),1);
            set(pHandle,'faceColor','none','EdgeColor',[0 0 0],'EdgeAlpha',1,'LineWidth',3);
            pFaceAlpha=.7;
            set(pHandle,'FaceAlpha',pFaceAlpha);
            pO=[2 3 1];
            
            for kk=1:6
                sq123{kk}=sq123{kk}(pO,:);
            end
            sPidx=sPidx(pO);
            sPidxB=sPidxB(pO);
        end
    elseif j==2
        for jj=1:3
            pHandle=patch(max(sq123{1}(1,:)*size(rPSD,1),1),max(sq123{1}(2,:)*size(rPSD,2),1),max(sq123{1}(3,:)*size(rPSD,3),1),1);
            set(pHandle,'faceColor','none','EdgeColor',[0 0 0],'EdgeAlpha',1);
            pHandle=patch(max(sq123{1}(1,:)*size(rPSD,1)+sPidxB(1)*size(rPSD,1),1),max(sq123{1}(2,:)*size(rPSD,2)+sPidxB(2)*size(rPSD,2),1),max(sq123{1}(3,:)*size(rPSD,3)+sPidxB(3)*size(rPSD,3),1),1);
            set(pHandle,'faceColor','none','EdgeColor',[0 0 0],'EdgeAlpha',1,'LineWidth',3);
            pHandle=patch(max(sq123{1}(1,:)*size(rPSD,1)+sPidxB(1)*size(rPSD,1),1),max(sq123{1}(2,:)*size(rPSD,2)+sPidxB(2)*size(rPSD,2),1),max(sq123{1}(3,:)*size(rPSD,3)+sPidxB(3)*size(rPSD,3),1),1);
            set(pHandle,'faceColor','none','EdgeColor',[0 0 0],'EdgeAlpha',1,'LineWidth',3);
            pFaceAlpha=.7;
            set(pHandle,'FaceAlpha',pFaceAlpha);
            pO=[2 3 1];
            
            for kk=1:6
                sq123{kk}=sq123{kk}(pO,:);
            end
            sPidx=sPidx(pO);
            sPidxB=sPidxB(pO);
        end
    end
    
    if j==1
        cbP1=get(ax1,'Position');
        cbP1=cbP1.*[1.5 1.5 1.5 1]+[-.1 -.2 0 0];
        set(ax1,'Position',cbP1);
    else
        cbP1=get(ax2,'Position');
        cbP1=cbP1.*[1.5 1.5 1.5 1]+[-.4 -.2 0 0];
        set(ax2,'Position',cbP1);
    end
    
    L={'$\xi_1$','$\xi_2$','$\xi_3$'};
    tHandle=text(0,0,size(rPSD,3)+.1,L{3},'Interpreter','Latex');
    set(tHandle,'fontsize',22);
    tHandle=text(0,size(rPSD,2)-.5,0,L{2},'Interpreter','Latex');
    set(tHandle,'fontsize',22);
    tHandle=text(size(rPSD,1)-.5,0,0,L{1},'Interpreter','Latex');
    set(tHandle,'fontsize',22);
    
    
    if j==1
        tHandle=text(0,-2,1.3*size(rPSD,3),'LLL corner','Interpreter','Latex');
        set(tHandle,'fontsize',18);
    else
        tHandle=text(0,-2,1.3*size(rPSD,3),'HHH corner','Interpreter','Latex');
        set(tHandle,'fontsize',18);
    end
    
    
    cbP=get(cb,'Position');
    cbP=cbP.*[.8 .8 .5 0.45]+[0.15 cbP(4)/3.5 0 0];
    set(cb,'Position',cbP);
    set(cb,'FontSize',16)
    %      set(gcf,'Renderer','zbuffer')
    
end

end









function visualize3DXsect( D )
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% visualize3DXsect visualizes a 3D cross-section of a 3D data
%-------------------------------------------------------------------------
%  EXAMPLE:
%
%       load mri;  % load 3D MRI data
%       D = double(squeeze(D));  % of size 128x128x27
%       visualize3DXsect( D ) ;  % visualization of a 3D cross-section
%-------------------------------------------------------------------------
%            Authors :         Nasser Eslahi & Alessandro Foi
%                                   Tampere University
%        Contact info:          firstname.lastname@tuni.fi
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if ndims(D)~=3
    error('Input must be 3-D.')
end

D = (D-min(D(:)))./(max(D(:))-min(D(:))); % normalizing the range of D into the range [0, 1]

% show 3-D cross-sections
%             h = figure;
%             set(h, 'color', 'white');
%             set(h, 'renderer', 'zbuffer');

    x = round(size(D)./2);
    
    p1 = squeeze(D(x(1),:,:));
    p2 = squeeze(D(:,x(2),:))';
    p3 = squeeze(D(:,:,x(3)));
    
    offset = 1;
    p1(:,1:x(3)-offset) = [];
    p1(1:x(2)-offset,:) = [];
    
    p3(:,1:x(2)-offset) = [];
    p3(1:x(1)-offset,:) = [];
    
    p2(:,1:x(1)-offset) = [];
    p2(1:x(3)-offset,:) = [];
    
    x(:) = offset;
    
    x1 = zeros(size(p1)) + x(1);
    x2 = zeros(size(p2)) + x(2);
    x3 = zeros(size(p3)) + x(3);
    [X1_1,X1_2] = meshgrid(1:size(p1,2),1:size(p1,1));
    [X2_1,X2_2] = meshgrid(1:size(p2,2),1:size(p2,1));
    [X3_1,X3_2] = meshgrid(1:size(p3,2),1:size(p3,1));
    hold all
    warp(X1_2,x1,X1_1,p1)
    warp(x2,X2_1,X2_2,p2)
    warp(X3_1,X3_2,x3,p3)
    view([35 30])
    axis off vis3d;
    camproj('persp')

end







function RGB_im = MS2RGB(x, WaveLengths)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% MS2RGB provides the RGB representation of the input multispectral image
%
%  FUNCTION INTERFACE:
%  bar_gray(rootPSD, txt, sigma_k)
% 
%  INPUTS:
%     1) x              (3D array) : the input 3D multispectral image
%     2) WaveLengths    (1D array) : the wavelengths of the input image 
%                                    (spectral bands interval)
%  Output:
%     1) RGB            (3D array) : the RGB representation of the input image
%-------------------------------------------------------------------------
%  EXAMPLE:
%
%  load('../Data/chart_and_stuffed_toy_ms_16bit.mat')  % the 512x512x31 "chart_and_stuffed_toy" 
%                                                      % multispectral image, available at:
%                                                      % http://www.cs.columbia.edu/CAVE/databases/multispectral/
%  WaveLengths       = 400:10:700;   % wavelengths
%
%  RGB_im = MS2RGB(x, WaveLengths);
%  figure, imshow(RGB_im.^.5)        % visualization (the power 0.5 is for gamma-correction)
%
%-------------------------------------------------------------------------
%            Authors :         Nasser Eslahi & Alessandro Foi
%                                   Tampere University
%        Contact info:          firstname.lastname@tuni.fi
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

R = mean(x(:,:,(WaveLengths<=700 & WaveLengths>=635) ), 3);
G = mean(x(:,:,(WaveLengths<=560 & WaveLengths>=520) ), 3);
B = mean(x(:,:,(WaveLengths<=490 & WaveLengths>=450) ), 3);

RGB_im = cat(3,R,G,B);
RGB_im =( RGB_im - min(RGB_im(:)) )./(max(RGB_im(:)) -  min(RGB_im(:)));

end







function hout = suptitle(str)
%SUPTITLE puts a title above all subplots.
%
%	SUPTITLE('text') adds text to the top of the figure
%	above all subplots (a "super title"). Use this function
%	after all subplot commands.
%
%   SUPTITLE is a helper function for yeastdemo.

%   Copyright 2003-2014 The MathWorks, Inc.


% Warning: If the figure or axis units are non-default, this
% function will temporarily change the units.

% Parameters used to position the supertitle.

% Amount of the figure window devoted to subplots
plotregion = .92;

% Y position of title in normalized coordinates
titleypos  = .95;

% Fontsize for supertitle
fs = get(gcf,'defaultaxesfontsize')+4;

% Fudge factor to adjust y spacing between subplots
fudge=1;

haold = gca;
figunits = get(gcf,'units');

% Get the (approximate) difference between full height (plot + title
% + xlabel) and bounding rectangle.

if ~strcmp(figunits,'pixels')
    set(gcf,'units','pixels');
    pos = get(gcf,'position');
    set(gcf,'units',figunits);
else
    pos = get(gcf,'position');
end
ff = (fs-4)*1.27*5/pos(4)*fudge;

% The 5 here reflects about 3 characters of height below
% an axis and 2 above. 1.27 is pixels per point.

% Determine the bounding rectangle for all the plots

h = findobj(gcf,'Type','axes');

oldUnits = get(h, {'Units'});
if ~all(strcmp(oldUnits, 'normalized'))
    % This code is based on normalized units, so we need to temporarily
    % change the axes to normalized units.
    set(h, 'Units', 'normalized');
    cleanup = onCleanup(@()resetUnits(h, oldUnits));
end

max_y=0;
min_y=1;
oldtitle = [];
numAxes = length(h);
thePositions = zeros(numAxes,4);
for i=1:numAxes
    pos=get(h(i),'pos');
    thePositions(i,:) = pos;
    if ~strcmp(get(h(i),'Tag'),'suptitle')
        if pos(2) < min_y
            min_y=pos(2)-ff/5*3;
        end
        if pos(4)+pos(2) > max_y
            max_y=pos(4)+pos(2)+ff/5*2;
        end
    else
        oldtitle = h(i);
    end
end

if max_y > plotregion
    scale = (plotregion-min_y)/(max_y-min_y);
    for i=1:numAxes
        pos = thePositions(i,:);
        pos(2) = (pos(2)-min_y)*scale+min_y;
        pos(4) = pos(4)*scale-(1-scale)*ff/5*3;
        set(h(i),'position',pos);
    end
end

np = get(gcf,'nextplot');
set(gcf,'nextplot','add');
if ~isempty(oldtitle)
    delete(oldtitle);
end
axes('pos',[0 1 1 1],'visible','off','Tag','suptitle');
ht=text(.5,titleypos-1,str);set(ht,'horizontalalignment','center','fontsize',fs);
set(gcf,'nextplot',np);
axes(haold);
if nargout
    hout=ht;
end

function resetUnits(h, oldUnits)
    % Reset units on axes object. Note that one of these objects could have
    % been an old supertitle that has since been deleted.
    valid = isgraphics(h);
    set(h(valid), {'Units'}, oldUnits(valid));
end

end



